﻿using System;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;

namespace BoundaryDemo
{
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
        }

        protected override void OnManipulationStarting(ManipulationStartingEventArgs args)
        {
            args.Mode = ManipulationModes.Translate;
            args.ManipulationContainer = mainGrid;
            args.Handled = true;
            base.OnManipulationStarting(args);
        }

        protected override void OnManipulationDelta(ManipulationDeltaEventArgs args)
        {
            FrameworkElement element = args.Source as FrameworkElement;
            MatrixTransform xform = element.RenderTransform as MatrixTransform;
            Matrix matx = xform.Matrix;
            Vector totalTranslate = args.DeltaManipulation.Translation;
            Vector usableTranslate = totalTranslate;

            if (args.IsInertial)
            {
                double xAttenuation = 0, yAttenuation = 0, attenuation = 0;

                if (matx.OffsetX < 0)
                    xAttenuation = -matx.OffsetX;
                else
                    xAttenuation = matx.OffsetX + element.ActualWidth - mainGrid.ActualWidth;

                if (matx.OffsetY < 0)
                    yAttenuation = -matx.OffsetY;
                else
                    yAttenuation = matx.OffsetY + element.ActualHeight - mainGrid.ActualHeight;

                xAttenuation = Math.Max(0, Math.Min(1, xAttenuation / (element.ActualWidth / 2)));
                yAttenuation = Math.Max(0, Math.Min(1, yAttenuation / (element.ActualHeight / 2)));
                attenuation = Math.Max(xAttenuation, yAttenuation);

                if (attenuation > 0)
                {
                    usableTranslate.X = (1 - attenuation) * totalTranslate.X;
                    usableTranslate.Y = (1 - attenuation) * totalTranslate.Y;

                    if (totalTranslate != usableTranslate)
                        args.ReportBoundaryFeedback(
                                new ManipulationDelta(totalTranslate - usableTranslate,
                                                      0, new Vector(), new Vector()));

                    if (attenuation > 0.99)
                        args.Complete();
                }
            }

            matx.Translate(usableTranslate.X, usableTranslate.Y);
            xform.Matrix = matx;

            args.Handled = true;
            base.OnManipulationDelta(args);
        }

        protected override void OnManipulationInertiaStarting(ManipulationInertiaStartingEventArgs args)
        {
            // Deceleration = 0.0001 DIUs / msec squared ~= 1 inch / second squared
            args.TranslationBehavior.DesiredDeceleration = 0.0001;

            base.OnManipulationInertiaStarting(args);
        }

        protected override void OnManipulationBoundaryFeedback(ManipulationBoundaryFeedbackEventArgs args)
        {
            //  Uncomment to eliminate automatic bounce-back
            //  args.Handled = true;

            base.OnManipulationBoundaryFeedback(args);
        }
    }
}
