﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;

namespace RotationalInertiaDemo
{
    public partial class MainWindow : Window
    {
        ValueSmoother valueSmoother = new ValueSmoother();
        double multiplier;

        public MainWindow()
        {
            InitializeComponent();
            radioDegreesPerMsec.IsChecked = true;
        }

        void OnUserControlManipulationStarting(object sender, ManipulationStartingEventArgs args)
        {
            UserControl ctrl = sender as UserControl;

            args.ManipulationContainer = (sender as FrameworkElement).Parent as UIElement;
            args.IsSingleTouchEnabled = true;
            args.Mode = ManipulationModes.Rotate;
            args.Pivot = new ManipulationPivot(new Point(ctrl.ActualWidth / 2, 
                                                         ctrl.ActualHeight / 2), 50);

            valueMeter.Value = multiplier * valueSmoother.AverageVelocities(0, true, false);
        }

        void OnUserControlManipulationDelta(object sender, ManipulationDeltaEventArgs args)
        {
            if (args.IsInertial)
            {
                valueMeter.Value = args.Velocities.AngularVelocity;
            }
            else
            {
                valueMeter.Value = multiplier *
                    valueSmoother.AverageVelocities(args.Velocities.AngularVelocity, false, false);
            }

            rotate.Angle += args.DeltaManipulation.Rotation;
        }

        void OnUserControlManipulationInertiaStarting(object sender, ManipulationInertiaStartingEventArgs args)
        {
            args.RotationBehavior.DesiredDeceleration = slider.Value;
        }

        void OnUserControlManipulationCompleted(object sender, ManipulationCompletedEventArgs args)
        {
            valueMeter.Value = multiplier * 
                valueSmoother.AverageVelocities(args.FinalVelocities.AngularVelocity, false, true);
        }

        void OnUnitsRadioButtonChecked(object sender, RoutedEventArgs args)
        {
            RadioButton radio = args.Source as RadioButton;
            UnitsInfo unitsInfo = radio.Tag as UnitsInfo;

            valueMeter.Minimum = unitsInfo.Minimum;
            valueMeter.Maximum = unitsInfo.Maximum;
            valueMeter.UnitsLabel = unitsInfo.UnitsLabel;
            valueMeter.TickValues = unitsInfo.TickValues;
            valueMeter.TickLabel = unitsInfo.TickLabel;

            multiplier = unitsInfo.Multiplier;
        }
    }
}
