﻿using System;
using System.Collections.Generic;

namespace RotationalInertiaDemo
{
    public class ValueSmoother
    {
        List<ValueAndTime> values = new List<ValueAndTime>();
        TimeSpan period = TimeSpan.FromSeconds(0.25);

        public double AverageVelocities(double velocity, bool start, bool finish)
        {
            double valueOut;

            if (start)
            {
                values.Clear();
                values.Add(new ValueAndTime(velocity, DateTime.Now));
                valueOut = velocity;
            }

            else if (finish)
            {
                valueOut = velocity;
            }

            else
            {
                DateTime dt = DateTime.Now;
                values.Add(new ValueAndTime(velocity, dt));
                double accumWeights = 0;
                double weightedValues = 0;
                int index = 0;

                while (index < values.Count)
                {
                    ValueAndTime valueAndTime = values[index];
                    TimeSpan elapsed = dt - valueAndTime.DateTime;

                    if (elapsed >= period)
                    {
                        values.RemoveAt(index);
                    }
                    else
                    {
                        double weight = 1 - elapsed.TotalMilliseconds / period.TotalMilliseconds;
                        accumWeights += weight;
                        weightedValues += valueAndTime.Value * weight;
                        index++;
                    }
                }

                valueOut = weightedValues / accumWeights;
            }

            return valueOut;
        }

        struct ValueAndTime
        {
            public double Value { set; get; }
            public DateTime DateTime { set; get; }

            public ValueAndTime(double value, DateTime dt)
                : this()
            {
                Value = value;
                DateTime = dt;
            }
        }
    }
}
